@@ -791,11 +791,14 @@ class DictMethodEntryHelper(object):
             logger.debug("entity type atom counts %r", self.__commonU.getEntityTypeHeavyAtomCounts(dataContainer, modelId=repModelId))
             logger.debug("instance atom counts %r", self.__commonU.getEntityTypeHeavyAtomCounts(dataContainer, modelId=repModelId))
             #
+
             if numHeavyAtomsModel > 0:
                 cObj.setValue(numHeavyAtomsModel, "deposited_atom_count", 0)
                 cObj.setValue(numModelsTotal, "deposited_model_count", 0)
+                tCD = self.__commonU.getEntityTypeHeavyAtomCounts(dataContainer, modelId=repModelId)
+                wCount = tCD["water"] if tCD and "water" in tCD else 0
+                cObj.setValue(wCount, "deposited_solvent_atom_count", 0)
             #
-
             # ---------------------------------------------------------------------------------------------------------
             #  Deposited monomer/residue instance counts
             #
